#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example demonstrates how to add a keyword to an ad group."""


import argparse
import sys
from flask import Flask
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException

import numpy as np
import xlrd
from openpyxl import load_workbook
import pandas as pd


def _add_keywords_to_adgroup(client, customer_id, ad_group_id, array_of_keywordarrays):

    #array_of_keywordarrays contain 3 arrays for broad keywords, phrase keywords and exact keywords
    #These are all for 1 campaign, don't confuse with campaign priority.
    keyword_array_broad = array_of_keywordarrays[0]
    keyword_array_phrase = array_of_keywordarrays[1]
    keyword_array_exact = array_of_keywordarrays[2]

    ad_group_service = client.get_service("AdGroupService")
    ad_group_criterion_service = client.get_service("AdGroupCriterionService")

    # Create keyword.
    operation = client.get_type("AdGroupCriterionOperation")
    criterion = operation.create
    # Initialising the adgroup keywords will be tied to by header customers/CUSTOMERID/adgroup/ADGROUPID
    criterion.ad_group = ad_group_service.ad_group_path(
        customer_id, ad_group_id
    )
    criterion.status = client.get_type(
        "AdGroupCriterionStatusEnum"
    ).AdGroupCriterionStatus.ENABLED

    # operations holds all the calls to be made to the api.
    operations = []

    for i in keyword_array_broad:
        word = str([i])
        operations.append(_create_negative_keyword_ad(client, customer_id, ad_group_id, word, "BROAD"))

    for i in keyword_array_phrase:
        word = str([i])
        operations.append(_create_negative_keyword_ad(client, customer_id, ad_group_id, word, "PHRASE"))

    for i in keyword_array_exact:
        word = str([i])
        operations.append(_create_negative_keyword_ad(client, customer_id, ad_group_id, word, "EXACT"))

    # Add keyword
    criterion_response = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id,
            operations=operations,
        )
    )

    # print(
    #     "Created keyword "
    #     f"{criterion_response.results[0].resource_name}."
    # )

def _add_keywords_to_campaign(client, customer_id, campaign_id, array_of_keywordarrays):

    #array_of_keywordarrays contain 3 arrays for broad keywords, phrase keywords and exact keywords
    #These are all for 1 campaign, don't confuse with campaign priority.
    keyword_array_broad = array_of_keywordarrays[0]
    keyword_array_phrase = array_of_keywordarrays[1]
    keyword_array_exact = array_of_keywordarrays[2]

    campaign_service = client.get_service("CampaignService")
    campaign_criterion_service = client.get_service("CampaignCriterionService")

    # Create keyword.
    operation = client.get_type("CampaignCriterionOperation")
    criterion = operation.create
    # Initialising the adgroup keywords will be tied to by header customers/CUSTOMERID/campaign/CAMPAIGNID
    criterion.campaign = campaign_service.campaign_path(
        customer_id, campaign_id
    )
    criterion.status = client.get_type(
        "CampaignCriterionStatusEnum"
    ).CampaignCriterionStatus.ENABLED

    # operations holds all the calls to be made to the api.
    operations = []

    for i in keyword_array_broad:
        word = str([i])
        word = word.strip("['']")
        #print(word)
        operations.append(_create_negative_keyword_campaign(client, customer_id, campaign_id, word, "BROAD"))

    for i in keyword_array_phrase:
        word = str([i])
        word = word.strip("['']")
        operations.append(_create_negative_keyword_campaign(client, customer_id, campaign_id, word, "PHRASE"))

    for i in keyword_array_exact:
        word = str([i])
        word = word.strip("['']")
        operations.append(_create_negative_keyword_campaign(client, customer_id, campaign_id, word, "EXACT"))

    # Add keyword
    criterion_response = (
        campaign_criterion_service.mutate_campaign_criteria(
            customer_id=customer_id,
            operations=operations,
        )
    )

    # print(
    #     "Created keyword "
    #     f"{criterion_response.results[0].resource_name}."
    # )


def _create_negative_keyword_ad (client, customer_id, ad_group_id, keyword_text, keyword_type):

    #Initialise ad group operation
    operation = client.get_type("AdGroupCriterionOperation")
    criterion = operation.create
    #Give adgroup path to assign keyword to.
    criterion.ad_group = client.get_service("AdGroupService").ad_group_path(customer_id, ad_group_id)
    criterion.status = client.get_type("AdGroupCriterionStatusEnum").AdGroupCriterionStatus.ENABLED

    #Assign keyword
    criterion.keyword.text = keyword_text
    criterion.negative = True

    #Assign match type
    if keyword_type == "EXACT":
        criterion.keyword.match_type = client.get_type(
            "KeywordMatchTypeEnum"
        ).KeywordMatchType.EXACT

    if keyword_type == "BROAD":
        criterion.keyword.match_type = client.get_type(
            "KeywordMatchTypeEnum"
        ).KeywordMatchType.BROAD

    if keyword_type == "PHRASE":
        criterion.keyword.match_type = client.get_type(
            "KeywordMatchTypeEnum"
        ).KeywordMatchType.PHRASE

    return operation

def _create_negative_keyword_campaign (client, customer_id, campaign_id, keyword_text, keyword_type):

    #Initialise ad group operation
    operation = client.get_type("CampaignCriterionOperation")
    criterion = operation.create
    #Give adgroup path to assign keyword to.
    criterion.campaign = client.get_service("CampaignService").campaign_path(customer_id, campaign_id)
    criterion.status = client.get_type("CampaignCriterionStatusEnum").CampaignCriterionStatus.ENABLED

    #Assign keyword
    criterion.keyword.text = keyword_text
    criterion.negative = True

    #Assign match type
    if keyword_type == "EXACT":
        criterion.keyword.match_type = client.get_type(
            "KeywordMatchTypeEnum"
        ).KeywordMatchType.EXACT

    if keyword_type == "BROAD":
        criterion.keyword.match_type = client.get_type(
            "KeywordMatchTypeEnum"
        ).KeywordMatchType.BROAD

    if keyword_type == "PHRASE":
        criterion.keyword.match_type = client.get_type(
            "KeywordMatchTypeEnum"
        ).KeywordMatchType.PHRASE

    return operation

def _importexcelfile(filename):
    """
    Imports Excel file and splits columns into 3 pandas series for irrelevant, broad and exact campaign.
    It also calls _merge_into_keyword_list which further separate them into 3 series depending on leading character
    for identification of keyword type (broad, phrase or exact keywords)
    It returns an array of arrays in the following structure:
    Array = [Irrelevant, Broad, Exact]
    where each of those items are another array of structure:
    Array2 = [KeywordIrrelevant, KeywordPhrase, KeywordExact]
    """

    filepath = f"uploads/{filename}"
    #Reads data from excel file into pandas dataframe.
    data = pd.read_excel(rf'{filepath}', sheet_name=0,
                         index_col=None, header=0, na_values=['NA'])
    #Get all values from first column, removing NA values
    irrelevant_column = data.iloc[:, 0]
    irrelevant_column = irrelevant_column.dropna()

    #Get all values from column 4 (all negatives) and remove na values
    allnegative_column = data.iloc[:,3]
    allnegative_column = allnegative_column.dropna()

    #Create df for broad column and drop na values.
    broad_column = data.iloc[:, 1]
    broad_column = broad_column.dropna()

    #Create df for broad column and drop na values.
    exact_column = data.iloc[:, 2]
    exact_column = exact_column.dropna()

    #merge irrelevant column and all negatives into one dataframe with one column.
    irrelevant_column_concat = [irrelevant_column, allnegative_column, broad_column, exact_column]
    irrelevant_column_concat = pd.concat(irrelevant_column_concat, ignore_index=True)
    irrelevantCamp_keyword_array = _merge_into_keyword_list(irrelevant_column_concat)

    #merge irrelevant column and all negatives into one dataframe with one column.
    broad_column_concat = [broad_column, allnegative_column, exact_column]
    broad_column_concat = pd.concat(broad_column_concat, ignore_index=True)
    broadCamp_keywords_array = _merge_into_keyword_list(broad_column_concat)

    #merge irrelevant column and all negatives into one dataframe with one column.
    exact_column_concat = [exact_column, allnegative_column]
    exact_column_concat = pd.concat(exact_column_concat, ignore_index=True)
    exactCamp_keywords_array = _merge_into_keyword_list(exact_column_concat)

    return irrelevantCamp_keyword_array, broadCamp_keywords_array, exactCamp_keywords_array


def _merge_into_keyword_list(campaign_keywords):

    #The character to look for to identify a phrase keyword.
    phrase_splitword = "\""
    exact_splitword = "["

    #Finding the entries in broad column concat that start wih " and place them in new series, removing na values from deleted entries.
    phrase = campaign_keywords.where(campaign_keywords.str.startswith(phrase_splitword)).dropna().astype(str)
    exact = campaign_keywords.where(campaign_keywords.str.startswith(exact_splitword)).dropna().astype(str)
    #Creating the broad keyword list by finding the entries that are not in the above two series. (with other words, removing the exact and phrase keywords)
    broad = campaign_keywords[~campaign_keywords.isin(phrase)].astype(str)
    #For the final bit unnecessary whitespaces as well as the quotation marks and square brackets are removed.
    broad = broad[~broad.isin(exact)].reset_index(drop=True).dropna().astype(str).str.strip()
    #Remove the quotations on both sides.
    phrase = phrase.str.replace('"', '').str.strip().str.strip('')
    #Remove square brackets on both sides.
    exact = exact.str.strip('[]').str.strip().str.strip('')
    #at this point you have broad, phrase, exact containing all the keywords with different match types for the campaign.

    return broad, phrase, exact


def _importexcelfile_test(filepath):
    """
    Imports Excel file and splits columns into 3 pandas series for irrelevant, broad and exact campaign.
    It also calls _merge_into_keyword_list which further separate them into 3 series depending on leading character
    for identification of keyword type (broad, phrase or exact keywords)
    It returns an array of arrays in the following structure:
    Array = [Irrelevant, Broad, Exact]
    where each of those items are another array of structure:
    Array2 = [KeywordIrrelevant, KeywordPhrase, KeywordExact]
    """

    #Reads data from excel file into pandas dataframe.
    data = pd.read_excel(rf'{filepath}', sheet_name=0,
                         index_col=None, header=0, na_values=['NA'])
    #Get all values from first column, removing NA values
    irrelevant_column = data.iloc[:, 0]
    irrelevant_column = irrelevant_column.dropna()

    #Get all values from column 4 (all negatives) and remove na values
    allnegative_column = data.iloc[:,3]
    allnegative_column = allnegative_column.dropna()

    #Create df for broad column and drop na values.
    broad_column = data.iloc[:, 1]
    broad_column = broad_column.dropna()

    #Create df for broad column and drop na values.
    exact_column = data.iloc[:, 2]
    exact_column = exact_column.dropna()

    #merge irrelevant column and all negatives into one dataframe with one column.
    irrelevant_column_concat = [irrelevant_column, allnegative_column]
    irrelevant_column_concat = pd.concat(irrelevant_column_concat, ignore_index=True)
    irrelevantCamp_keyword_array = _merge_into_keyword_list(irrelevant_column_concat)

    #merge irrelevant column and all negatives into one dataframe with one column.
    broad_column_concat = [broad_column, allnegative_column]
    broad_column_concat = pd.concat(broad_column_concat, ignore_index=True)
    broadCamp_keywords_array = _merge_into_keyword_list(broad_column_concat)

    #merge irrelevant column and all negatives into one dataframe with one column.
    exact_column_concat = [exact_column, allnegative_column]
    exact_column_concat = pd.concat(exact_column_concat, ignore_index=True)
    exactCamp_keywords_array = _merge_into_keyword_list(exact_column_concat)

    return irrelevantCamp_keyword_array, broadCamp_keywords_array, exactCamp_keywords_array




if __name__ == "__main__":
    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    yaml_filepath = '//DDSERVER/Downloads$/googleadscode/googleads.yaml'
    customer_id = '5927843196'
    merchant_center_account_id = 107945304
    googleads_client = GoogleAdsClient.load_from_storage(yaml_filepath, version="v6")
    ad_group_id = "116166437250"

    data = _importexcelfile_test(r'C:\Users\malin.sundstrom\Downloads\Parasolnegativekeywords.xlsx')
    #_importexcelfile returns 3 arrays of arrays with keywords for irrelevant campaign, broad and exact campaign.


    # try:
    #     _add_keywords_to_campaign(
    #     googleads_client, customer_id, ad_group_id, data_irrelevantcamp
    # )
    # except GoogleAdsException as ex:
    #     print(
    #         f'Request with ID "{ex.request_id}" failed with status '
    #         f'"{ex.error.code().name}" and includes the following errors:'
    #     )
    #     for error in ex.failure.errors:
    #         print(f'	Error with message "{error.message}".')
    #         if error.location:
    #             for field_path_element in error.location.field_path_elements:
    #                 print(f"\t\tOn field: {field_path_element.field_name}")
    #     sys.exit(1)
